#include <stdio.h>
#include <stdlib.h>
#include <math.h>

/*
	This program is based on DEMON-5, written by my father,
	Jørgen Kjær at Haldor Topsøe.

	This is translated from GIER ALGOL 4 into C by Mogens Kjær, mk@lemo.dk
*/

static long MODUL=10000000000L;
static int linerest,decimals,limit,count,D,E,c39;
static long cell,carry,cell2;

static int CHAR;
static int cstate,ctype;

static char cbuf[100];
static int cpnt=0;

static void clearbuf()
{
  cpnt=0;
}

static void lyn()
{
  CHAR=getchar();
  if(cpnt<(sizeof(cbuf)-1)) cbuf[cpnt++]=CHAR;

  if(CHAR==EOF)
  {
    printf("\nEOF met\n");
    exit(0);
  }

  if(CHAR>='0' && CHAR<='9')
    ctype=1;
  else if(CHAR==' ')
    ctype=0;
  else if(CHAR==',' || CHAR==';' || CHAR=='\n')
    ctype=2;
  else if(CHAR=='+' || CHAR=='-')
    ctype=3;
  else if(CHAR=='e' || CHAR=='E' || CHAR=='\'')
    ctype=5;
  else if(CHAR=='.')
    ctype=4;
  else
    ctype=2;
}

static long readinteger()
{
  /*
    ctype:

  0:	whitespace
  1:	digit
  2:	terminator
  3:	sign
  4:	.
  5:	exponent

    cstate:
  0:	before
  1:	after sign
  2:	after digit

    caction:
  0:	nothing
  1:	store sign
  2:	store digit
  3:	done
  4:	error
  */
  /*        state ctype */
  int cstates[3][6] =
    {
  /*  sp   d   ,   ±   .   e */
     { 0,  2,  0,  1,  0,  0},  /* before */
     { 0,  2,  9,  9,  9,  9},  /* after sign */
     { 0,  2,  9,  9,  9,  9}   /* after digit */
    };
  int cactions[3][6] =
    {
  /*  sp   d   ,   ±   .   e */
     { 0,  2,  0,  1,  0,  0},  /* before */
     { 0,  2,  4,  4,  4,  4},  /* after sign */
     { 0,  2,  3,  4,  4,  4}   /* after digit */
    };
  int sign1,action;
  long num1;
  cstate=0;
  sign1=0;
  num1=0;
  clearbuf();
  while(1)
  {
    lyn();
    action=cactions[cstate][ctype];
    cstate=cstates[cstate][ctype];
    switch(action)
    {
      case 0: /* Nothing */
	break;
      case 1: /* store sign */
	sign1=CHAR=='-';
	break;
      case 2: /* store digit */
	num1=num1*10+CHAR-'0';
	break;
      case 3: /* done */
	if(sign1)num1=-num1;
	return num1;
      case 4: /* error */
	cbuf[cpnt]=0;
	printf("Error in integer number: %s\n",cbuf);
	exit(0);
    }
  }
}

static double readreal()
{
  /*
    ctype:

  0:	whitespace
  1:	digit
  2:	terminator
  3:	sign
  4:	.
  5:	exponent

    cstate:
  0:	before
  1:	after sign
  2:	after digit, before .
  3:	after .
  4:	after e
  5:	after e sign

    caction:
  0:	nothing
  1:	store sign
  2:	store digit before .
  3:	store digit after .
  4:	store exponent sign
  5:	store exponent digit
  6:	done
  7:	error
  */
  /*        state ctype */
  int cstates[6][6] =
    {
  /*  sp   d   ,   ±   .   e */
     { 0,  2,  0,  1,  3,  4},  /* before */
     { 0,  2,  9,  3,  5,  4},  /* after sign */
     { 0,  2,  9,  9,  3,  4},  /* after digit */
     { 0,  3,  9,  9,  9,  4},  /* after . */
     { 0,  5,  9,  5,  9,  9},  /* after e */
     { 0,  5,  9,  9,  9,  9}   /* after e sign */
    };
  int cactions[6][6] =
    {
  /*  sp   d   ,   ±   .   e */
     { 0,  2,  0,  1,  0,  0},  /* before */
     { 0,  2,  7,  7,  0,  0},  /* after sign */
     { 0,  2,  6,  7,  0,  0},  /* after digit */
     { 0,  3,  6,  7,  7,  0},  /* after . */
     { 0,  5,  6,  4,  7,  7},  /* after e */
     { 0,  5,  6,  7,  7,  7}   /* after e sign */
    };
  int sign1,sign2,expo,ndec,action;
  double num1;
  cstate=0;
  sign1=0;
  sign2=0;
  expo=0;
  num1=0;
  ndec=0;

  clearbuf();
  while(1)
  {
    lyn();
    action=cactions[cstate][ctype];
    cstate=cstates[cstate][ctype];
    switch(action)
    {
      case 0: /* Nothing */
	break;
      case 1: /* store sign */
	sign1=CHAR=='-';
	break;
      case 3: /* store digit after . */
	ndec++;
      case 2: /* store digit before . */
	num1=num1*10+CHAR-'0';
	break;
      case 4: /* store exponent sign */
	sign2=CHAR=='-';
	break;
      case 5: /* store exponent digit */
	expo=expo*10+CHAR-'0';
	break;
      case 6: /* done */
	if(sign2) expo=-expo;
	num1=num1*pow(10.0,((double)(expo-ndec)));
	if(sign1)num1=-num1;
	return num1;
      case 7: /* error */
	cbuf[cpnt]=0;
	printf("Error in real number: %s\n",cbuf);
	exit(0);
    }
  }
}

static void NEWPAGE()
{
  linerest=69;
  printf("\n");
//printf("\f\n");
}

static void LINE()
{
  if(linerest<8)
  {
    NEWPAGE();
  }
  else
  {
    linerest--;
    printf("\n");
  }
}

static void ACCEPT(int cond)
{
  if(!cond)
  {
    LINE();
    printf("Brug flere heltalscifre");
    exit(-1);
  }
}

static void ALARM(char *text)
{
  LINE();
  printf("Error in: %s",text);
  exit(-1);
}

static void PRINT(long *A, int *asize)
{
  int first,localasize;
  long DIVISOR,digit;
  int i,group,c;
  char space;
  void GROUP(long n)
  {
    DIVISOR=MODUL/10;
    space=first?32:48;
    for(i=1;i<=10;i++)
    {
      digit=n/DIVISOR;
      n=n%DIVISOR;
      if(digit!=0)
      {
	printf("%c", (char)(digit+48));
	first=0;
	space=48;
      }
      else
	printf("%c", space);
      if(i==5) printf(" ");
      DIVISOR=DIVISOR/10;
    }
  }
  first=1;
  group=0;
  LINE();
  localasize=((*asize)<0)?0:(*asize);
  c=localasize-decimals;
  for(count=localasize;count>=decimals;count--)
  {
    GROUP(A[c]);
    if(count==0 && decimals<0)
    {
      printf(".");
      first=0;
    }
    else
      printf(" ");
    group++;
    if( ((group%6)==0) && count != decimals) LINE();
    c--;
  }
}

static void ASSIGN(double x, long *A, int *asize)
{
  int c1,c2;
  long cell1,cell2;
  double factor;
  x=fabs(x);
  c1=c39;
  for(count=0; count<=c1; count++) A[count]=0;
  if(x==0)
  {
    (*asize) = 0;
  }
  else
  {
    (*asize) = floor(M_LOG10E/10.0*log(x));
    if((*asize)>limit) ALARM("ASSIGN");
    factor=pow((double)MODUL,(double) (*asize));
    cell1 = floor(x/factor);
    cell2 = floor((x/factor-cell1)*MODUL);
    c1=(*asize)-decimals;
    c2=c1-1;
    if(c2<0)
    {
      c2=c1;
      cell2=cell1;
    }
    if(c1<0)
    {
      c1=0;
      c2=0;
      cell1=0;
      cell2=0;
    }
    A[c1]=cell1;
    A[c2]=cell2;
  }
}

static void MULT(long *A, int *asize, long n)
{
  int c,c1;
  carry=0;
  c=0;
  c1=limit-decimals;
  for(count=0;count<=c1;count++)
  {
    cell=A[c];
//  RM=(carry+cell*n);
//  carry=RM/MODUL;
//  cell=RM%MODUL;
    __asm__(

	"movq %0,%%rax\n"
	"imulq %3\n"
	"addq %1,%%rax\n"
	"adcq $0,%%rdx\n"
	"idivq %2\n"
	"movq %%rdx,%0\n"
	"movq %%rax,%1\n"

	: "+rm" (cell), "+rm" (carry)
	: "rm" (MODUL), "rm" (n)
	: "rdx", "rax"

	);

    A[c]=cell;
    c++;
    if(count==((*asize) - decimals))
    {
      if(carry==0) return;
      else if(count<c1) (*asize) ++;
      else ALARM("MULT");
    }
  }
}

static void DIVIDE(long *A, int *asize, long n, int *empty)
{
  int c,first;
  first=1;
  carry=0;
  c=(*asize)-decimals;
  for(count=(*asize); count>=decimals; count--)
  {
    cell=A[c];
//  RM=(cell+carry*MODUL);
//  cell=RM/n;
//  carry=RM%n;
    __asm__(

	"movq %1,%%rax\n"
	"imulq %2\n"
	"addq %0,%%rax\n"
	"adcq $0,%%rdx\n"
	"idivq %3\n"
	"movq %%rdx,%1\n"
	"movq %%rax,%0\n"

	: "+rm" (cell), "+rm" (carry)
	: "rm" (MODUL), "rm" (n)
	: "rdx", "rax"

	);
    A[c]=cell;
    c--;
    if(first)
    {
      if(cell>0) first=0;
      else
      {
	if((*asize)>decimals) (*asize)--;
      }
    }
  }
  (*empty) = first && (cell==0);
}

static void COPY(long *A, int *asize, long *B, int *bsize)
{
  int c,c1;
  c1=c39;
  for(c=0;c<=c1;c++) B[c]=A[c];
  (*bsize)=(*asize);
}

static void ADD(long *B, int *bsize, int factor, long *A, int *asize)
{
  int c,i;
  c=-1;
  carry=0;
  for(count=decimals; count<=limit; count++)
  {
    c++;
    cell = A[c]+factor*B[c] + carry;
    carry=0;
    while(cell<0)
    {
      cell+=MODUL;
      carry--;
    }
    cell2=cell/MODUL;
    A[c]=cell-cell2*MODUL;
    carry+=cell2;
    if((count>=(*bsize))&&(carry==0)) break;
  }
  if(carry!=0) ALARM("ADD");
  (*asize) = limit;
  c=limit-decimals;
  while( (A[c--]==0) && ((*asize)>decimals))
  {
    (*asize)--;
  }
}

static void TESTPRINT(long *A, int *asize)
{
  int i,started;
  started=0;
  printf("\n%d  ",(*asize));
  for(i=c39;i>=0;i--)
  {
    if(A[i]!=0) started=1;
    if(started) printf(" %010ld",A[i]);
  }
}
static void LONGMULT(long *A, int *asize, long *B, int *bsize, long *C, int *csize)
{
  int c,c2, rsize, shift,s,s1,s2,s3,i;
  long factor;
  long *RES;
  RES=(long *) malloc(sizeof(long)*(c39+1));
  ASSIGN(0.0, C, csize);
  for(c=0; c<=((*bsize)-decimals); c++)
  {
    shift=c+decimals;
    factor=B[c];
    COPY(A,asize,RES,&rsize);
    MULT(RES,&rsize,factor);
    if(shift!=0)
    {
      s1=(shift<0)?(-c):(limit-decimals);
      s2=(shift<0)?(1):(-1);
      s3=(shift<0)?(limit-decimals):(-c);

      for(s=s1;(s*s2)<=(s3*s2);s+=s2)
      {
	if((s<0)||(s>(limit-decimals)))
	  cell=0;
	else
	  cell=RES[s];
	c2=s+shift;
	if(c2>(limit-decimals))
	{
	  if(cell!=0) ALARM("LONGMULT");
	}
	else
	{
	  if(c2>=0)
	  {
	    RES[c2]=cell;
	  }
	}
      }
    }
    rsize += shift;
    ADD(RES, &rsize, 1, C, csize);
  }
  free((char *) RES);
}

static void EXP(long *X, int *xsize, long *A, int *asize, long *XN, int *xnsize)
{
  int out,tsize;
  long *TERM,m;
  TERM=(long *) malloc(sizeof(long)*(c39+1));
  ASSIGN(1.0, A, asize);
  COPY(X, xsize, TERM, &tsize);
  ADD(X, xsize, 1, A, asize);
  out=0;
  m=1;
  while(!out)
  {
    m++;
    LONGMULT(X,xsize,TERM,&tsize,XN,xnsize);
    COPY(XN,xnsize,TERM,&tsize);
    DIVIDE(TERM,&tsize,m,&out);
    ADD(TERM,&tsize,1,A,asize);
  }
  free((char *) TERM);
}

static void EXPi(long i, long *A, int *asize, long *TERM, int *tsize)
{
  int out;
  long m;
  ASSIGN(1.0+((double)i), A, asize);
  ASSIGN(((double)i), TERM, tsize);
  out=0;
  m=1;
  while(!out)
  {
    m++;
    MULT(TERM,tsize,i);
    DIVIDE(TERM,tsize,m,&out);
    ADD(TERM,tsize,1,A,asize);
  }
}

static void ETO(long *A, int *asize, long *TERM, int *tsize)
{
  int out;
  long m;
  ASSIGN(2.0, A, asize);
  ASSIGN(1.0, TERM, tsize);
  out=0;
  m=1;
  while(!out)
  {
    m++;
    DIVIDE(TERM,tsize,m,&out);
    ADD(TERM,tsize,1,A,asize);
  }
}

static void PITO(long *A, int *asize, long *T2, int *t2size, long *T3, int *t3size)
{
  int out1,out2,out3,out;
  long m,*SUM,*T1;
  int factor,ssize,t1size;
  T1=(long *) malloc(sizeof(long)*(c39+1));
  SUM=(long *) malloc(sizeof(long)*(c39+1));
  ASSIGN(0.0, A, asize);
  ASSIGN(3.0, T1, &t1size);
  out1 = 0;
  ASSIGN(24.0,T2,t2size);
  DIVIDE(T2,t2size,171,&out2);
  ASSIGN(24.0,T3,t3size);
  DIVIDE(T3,t3size,1434,&out3);
  factor=-1;
  m=-1;
  while(!out1)
  {
    m+=2;
    ASSIGN(0.0,SUM,&ssize);
    ADD(T1,&t1size,1,SUM,&ssize);
    if(!out2) ADD(T2,t2size,1,SUM,&ssize);
    if(!out3) ADD(T3,t3size,1,SUM,&ssize);
    DIVIDE(SUM,&ssize,m,&out);
    factor=-factor;
    ADD(SUM,&ssize,factor,A,asize);
    DIVIDE(T1,&t1size,64,&out1);
    if(!out2) DIVIDE(T2,t2size,3249,&out2);
    if(!out3) DIVIDE(T3,t3size,57121,&out3);
  }
  free((char *) SUM);
  free((char *) T1);
}

static void SQRT(double x, long *A, int *asize, long *B, int *bsize, long *C, int *csize)
{
  int empty;
  int xsize,zsize,i,imax;
  long *X,*Z;
  X=(long *) malloc(sizeof(long)*(c39+1));
  Z=(long *) malloc(sizeof(long)*(c39+1));
  ASSIGN(x, X, &xsize);
  ASSIGN(sqrt(x), A, asize);
  ASSIGN(1.0/sqrt(x), Z, &zsize);
//A[0]=0;
//A[1]=4142135600L;
//A[2]=1;
//Z[0]=0;
//Z[1]=7071067824L;
//Z[2]=0;
  imax=((*asize)>zsize)?(*asize):zsize;
  imax=imax-decimals+1;
//imax=100;
//imax+=10;
  for(i=1;i<=imax;i++)
  {
    LONGMULT(A, asize, Z, &zsize, B, bsize);
    ASSIGN(2.0, C, csize);
    ADD(B, bsize, -1, C, csize);
    LONGMULT(Z, &zsize, C, csize, B, bsize);
    LONGMULT(B, bsize, X, &xsize, C, csize);
    COPY(B, bsize, Z, &zsize);
    ADD(C, csize, 1, A, asize);
    DIVIDE(A, asize, 2, &empty);
  }
  free((char *) Z);
  free((char *) X);
}

static void EXPISQN(long N, long *A, int *asize, long *B, int *bsize, long *C, int *csize)
{
  int xsize;
  long *X;
  X=(long *) malloc(sizeof(long)*(c39+1));
  PITO(A, asize, B, bsize, C, csize);
  SQRT((double)N, B, bsize, C, csize, X, &xsize);
  LONGMULT(A, asize, B, bsize, C, csize);
  EXP(C, csize, A, asize, B, bsize);
  free((char *) X);
}

static void FACTAB(long from, long step, long to, long *A, int *asize)
{
  long N,n;
  ACCEPT(limit>1+0.05*to*log((double)to));
  ASSIGN(1.0,A,asize);
  for(N=2;N<from;N++)
    MULT(A,asize,N);
  n=step-1;
  for(N=from; N<=to; N++)
  {
    MULT(A,asize,N);
    n++;
    if(n==step)
    {
      n=0;
      LINE();
      printf("N: %d, FAC(N): ",N);
      PRINT(A,asize);
    }
  }
}

static void POWTAB1(long from, long step, long to, long a, long *A, int *asize)
{
  long N,n;
  ACCEPT(limit>1+0.05*to*log((double)a));
  LINE();
  printf("a: %ld",a);
  ASSIGN(1.0, A, asize);
  for(N=1;N<from;N++)
    MULT(A,asize,a);
  n=step-1;
  for(N=from;N<=to;N++)
  {
    MULT(A,asize,a);
    n++;
    if(n==step)
    {
      n=0;
      LINE();
      printf("N : %d, a|&N: ",N);
      PRINT(A,asize);
    }
  }
}

static void POWTAB2(long from, long step, long to, long b, long *A, int *asize)
{
  long N,n;
  ACCEPT(limit>1+0.05*((double)b)*log((double)to));
  LINE();
  printf("b: %d",b);
  for(N=from;N<=to;N+=step)
  {
    ASSIGN(1.0, A, asize);
    for(n=1;n<=b;n++)
      MULT(A,asize,N);
    LINE();
    printf("N: %d, N|&b: ",N);
    PRINT(A,asize);
  }
}

static void ISOM(long N, long *PRI, int *psize, long *SEC, int *ssize,
                 long *TER, int *tsize)
{
  int *totsize,usize,vsize;
  long **TOT,*U,*V;
  int k,n,m,i,j,q,s,empty;

  totsize = (int *) malloc(sizeof(int)*(N+1));
  TOT = (long **) malloc(sizeof(long *)*(N+1));
  for(i=0;i<=N;i++) TOT[i]=(long *) malloc(sizeof(long)*(c39+1));
  U=(long *)malloc(sizeof(long)*(c39+1));
  V=(long *)malloc(sizeof(long)*(c39+1));

  ASSIGN(1.0, TOT[0], &totsize[0]);
  for(n=1;n<=N;n++)
  {
    ASSIGN(0.0,SEC,ssize);
    ASSIGN(0.0,TER,tsize);
    m=(n-1)/2;
    for(i=1;i<=m;i++)
    {
      j=n-1-i;
      if(i<j)
      {
	LONGMULT(TOT[i],&totsize[i],TOT[j],&totsize[j],V,&vsize);
      }
      else
      {
	ASSIGN(1.0,U,&usize);
	ADD(TOT[j],&totsize[j],1,U,&usize);
	LONGMULT(TOT[i],&totsize[i],U,&usize,V,&vsize);
	DIVIDE(V,&vsize,2,&empty);
      }
      ADD(V,&vsize,1,SEC,ssize);
    }
    m=(n-2)/2;
    for(i=1;i<=m;i++)
    {
      j=n-1-2*i;
      ASSIGN(i!=j?0.0:2.0,U,&usize);
      ADD(TOT[j],&totsize[j],1,U,&usize);
      LONGMULT(TOT[i],&totsize[i],U,&usize,V,&vsize);
      ADD(U,&usize,1,V,&vsize);
      LONGMULT(V,&vsize,TOT[i],&totsize[i],U,&usize);
      DIVIDE(U,&usize,(i!=j)?2.0:6.0,&empty);
      ADD(U,&usize,1,TER,tsize);
    }
    m=(n-4)/3;
    for(i=1;i<=m;i++)
    {
      q=(n-2-i)/2;
      for(j=i+1;j<=q;j++)
      {
	k=n-1-i-j;
	LONGMULT(TOT[j],&totsize[j],TOT[k],&totsize[k],V,&vsize);
	LONGMULT(V,&vsize,TOT[i],&totsize[i],U,&usize);
	ADD(U,&usize,1,TER,tsize);
      }
    }
    LINE();
    printf("N: %d",n);
    LINE();
    printf("PRI(N):");
    PRINT(TOT[n-1],&totsize[n-1]);
    LINE();
    printf("SEC(N):");
    PRINT(SEC,ssize);
    LINE();
    printf("TER(N):");
    PRINT(TER,tsize);
    LINE();
    ADD(TER, tsize, 1, SEC, ssize);
    ADD(TOT[n-1],&totsize[n-1],1,SEC,ssize);
    COPY(SEC,ssize,TOT[n],&totsize[n]);
  }
  free((char *) V);
  free((char *) U);
  for(i=0;i<=N;i++) free((char *)(TOT[i]));
  free((char *) TOT);
  free((char *) totsize);
}

static void test1()
{
  int i,asize,bsize,csize,out1;
  long *A,*B,*C;

  D=0;
  E=4000;
  decimals=D;
  if(decimals>0) decimals= -((decimals-1)/10+1);
  limit=(E-1)/10;
  c39=limit-decimals;
  A=(long *) malloc(sizeof(long)*(c39+1));
  B=(long *) malloc(sizeof(long)*(c39+1));
  C=(long *) malloc(sizeof(long)*(c39+1));
  ASSIGN(1.0, A, &asize);
  for(i=2; i<=500; i++)
  {
    MULT(A, &asize, (long) i);
    LINE();
    printf("N: %d:",i);
    PRINT(A, &asize);
  }
  for(i=2; i<=500; i++)
  {
    DIVIDE(A, &asize, (long) i, &out1);
    LINE();
    printf("N: %d:",i);
    PRINT(A, &asize);
  }
  LINE();
  free((char *) C);
  free((char *) B);
  free((char *) A);
}

static void test2()
{
  int i,asize,bsize,csize,out1;
  long *A,*B,*C;

  D=20000;
  E=3000;
  decimals=D;
  if(decimals>0) decimals= -((decimals-1)/10+1);
  limit=(E-1)/10;
  c39=limit-decimals;
  A=(long *) malloc(sizeof(long)*(c39+1));
  B=(long *) malloc(sizeof(long)*(c39+1));
  C=(long *) malloc(sizeof(long)*(c39+1));
  PITO(A,&asize,B,&bsize,C,&csize);
  PRINT(A,&asize);
  ETO(A,&asize,B,&bsize);
  PRINT(A,&asize);
  LINE();
  free((char *) C);
  free((char *) B);
  free((char *) A);
}

static void test3()
{
  int i,asize,bsize,csize,out1;
  long *A,*B,*C;

  D=0;
  E=400000;
  decimals=D;
  if(decimals>0) decimals= -((decimals-1)/10+1);
  limit=(E-1)/10;
  c39=limit-decimals;
  A=(long *) malloc(sizeof(long)*(c39+1));
  B=(long *) malloc(sizeof(long)*(c39+1));
  C=(long *) malloc(sizeof(long)*(c39+1));
  ASSIGN(1.0, A, &asize);
  for(i=2; i<=50000; i++)
  {
    MULT(A, &asize, (long) i);
  }
  LINE();
  PRINT(A, &asize);
  LINE();
  free((char *) C);
  free((char *) B);
  free((char *) A);
}

static void test4()
{
  int i,asize,bsize,csize,out1;
  long *A,*B,*C;

  for(D=10; D<=1000; D+=10)
  {
    E=3000;
  decimals=D;
  if(decimals>0) decimals= -((decimals-1)/10+1);
  limit=(E-1)/10;
  c39=limit-decimals;
  A=(long *) malloc(sizeof(long)*(c39+1));
  B=(long *) malloc(sizeof(long)*(c39+1));
  C=(long *) malloc(sizeof(long)*(c39+1));
  EXPi(700,A,&asize,B,&bsize);
  PRINT(A,&asize);
  LINE();
  free((char *) C);
  free((char *) B);
  free((char *) A);
  }
}

static void test5()
{
  int i,asize,bsize,csize,out1;
  long *A,*B,*C;

  for(D=10; D<=200; D+=10)
  {
  E=30;
  decimals=D;
  if(decimals>0) decimals= -((decimals-1)/10+1);
  limit=(E-1)/10;
  c39=limit-decimals;
  A=(long *) malloc(sizeof(long)*(c39+1));
  B=(long *) malloc(sizeof(long)*(c39+1));
  C=(long *) malloc(sizeof(long)*(c39+1));
  SQRT(2.0, A, &asize, B, &bsize, C, &csize);
  PRINT(A,&asize);
  LONGMULT(A,&asize,A,&asize,C,&csize);
  PRINT(C,&csize);
  LINE();
  free((char *) C);
  free((char *) B);
  free((char *) A);
  }
}

static void test6()
{
  int i,asize,bsize,csize,out1,N;
  long *A,*B,*C;

  D=200;
  E=200;
  decimals=D;
  if(decimals>0) decimals= -((decimals-1)/10+1);
  limit=(E-1)/10;
  c39=limit-decimals;
  A=(long *) malloc(sizeof(long)*(c39+1));
  B=(long *) malloc(sizeof(long)*(c39+1));
  C=(long *) malloc(sizeof(long)*(c39+1));
  for(N=1; N<=163; N++)
  {
    printf("\nN: %d",N);
    EXPISQN(N,A,&asize,B,&bsize,C,&csize);
    PRINT(A,&asize);
  }
  free((char *) C);
  free((char *) B);
  free((char *) A);
}

static void test7()
{
  double d;
  for(;;)
  {
    d=readinteger();
    printf("%.14g\n",d);
  }
}

static long next()
{
  long n;
  n=readinteger();
  LINE();
  printf("r := %d",n);
  return n;
}

static double nextr()
{
  double n;
  n=readreal();
  LINE();
  printf("r := %lg",n);
  return n;
}

static void CALCULATE()
{
  int i,asize,bsize,csize,out1,N,TYPE;
  long *A,*B,*C,p1,p2,p3,p4;

  for(;;)
  {
    D=readinteger();
    if(D<0) return;
    E=readinteger();
    LINE();
    printf("D: %d",D);
    LINE();
    printf("E: %d",E);
    decimals=D;
    if(decimals>0) decimals= -((decimals-1)/10+1);
    limit=(E-1)/10;
    c39=limit-decimals;
    A=(long *) malloc(sizeof(long)*(c39+1));
    B=(long *) malloc(sizeof(long)*(c39+1));
    C=(long *) malloc(sizeof(long)*(c39+1));
    for(;;)
    {
      LINE();
      LINE();
      TYPE=readinteger();
      printf("No: %d",TYPE);
      switch(TYPE)
      {
	case 1:
	  printf(" A := r;");
	  ASSIGN(nextr(),A,&asize);
	  break;
	case 2:
	  printf(" write(A);");
	  PRINT(A,&asize);
	  break;
	case 3:
	  printf(" B := A;");
	  COPY(A,&asize,B,&bsize);
	  break;
	case 4:
	  printf(" C := A;");
	  COPY(A,&asize,C,&csize);
	  break;
	case 5:
	  printf(" A := B;");
	  COPY(B,&bsize,A,&asize);
	  break;
	case 6:
	  printf(" C := B;");
	  COPY(B,&bsize,C,&csize);
	  break;
	case 7:
	  printf(" A := C;");
	  COPY(C,&csize,A,&asize);
	  break;
	case 8:
	  printf(" B := C;");
	  COPY(C,&csize,B,&bsize);
	  break;
	case 9:
	  printf(" A := A + B;");
	  ADD(B, &bsize, 1, A, &asize);
	  break;
	case 10:
	  printf(" A := A - B;");
	  ADD(B, &bsize, -1, A, &asize);
	  break;
	case 11:
	  printf(" A := A*r;");
	  MULT(A,&asize,next());
	  break;
	case 12:
	  printf(" A := A/r;");
	  DIVIDE(A,&asize,next(),&out1);
	  break;
	case 13:
	  printf(" C := A*B;");
	  LONGMULT(A,&asize,B,&bsize,C,&csize);
	  break;
	case 14:
	  printf(" A := PI;");
	  PITO(A,&asize,B,&bsize,C,&csize);
	  break;
	case 15:
	  printf(" A := exp(B);");
	  EXP(B,&bsize,A,&asize,C,&csize);
	  break;
	case 16:
	  printf(" A := sqrt(r);");
	  SQRT(nextr(),A,&asize,B,&bsize,C,&csize);
	  break;
	case 17:
	  printf(" A := exp(PI*sqrt(r));");
	  EXPISQN(next(),A,&asize,B,&bsize,C,&csize);
	  break;
	case 18:
	  printf(" FACTORIAL TABLE(r, r, r);");
	  /* Don't put the next()'s into the parameter list - they are called backwards */
	  p1=next();
	  p2=next();
	  p3=next();
	  FACTAB(p1,p2,p3,A,&asize);
	  break;
	case 19:
	  printf(" POWER TABLE(r, r, r, r|&variable);");
	  p1=next();
	  p2=next();
	  p3=next();
	  p4=next();
	  POWTAB1(p1,p2,p3,p4,A,&asize);
	  break;
	case 20:
	  printf(" POWER TABLE(r, r, r, variable|&r);");
	  p1=next();
	  p2=next();
	  p3=next();
	  p4=next();
	  POWTAB2(p1,p2,p3,p4,A,&asize);
	  break;
	case 21:
	  printf(" ISOMER TABLE(r);");
	  ISOM(next(),A,&asize,B,&bsize,C,&csize);
	  break;
	case 22:
	  printf(" stop");
	  goto EXIT1;
	case 23:
	  printf(" A := ETO;");
	  ETO(A,&asize,B,&bsize);
	  break;
      }
    }
EXIT1:
    free((char *) C);
    free((char *) B);
    free((char *) A);
  }
}



main(int argc, char **argv)
{

  linerest=69;
//test1();
//test2();
//test3();
//test4();
//test5();
//test6();
//test7();
  CALCULATE();
  LINE();
}

